@@ -37,8 +37,12 @@
 #define	USB_PID_FSF_EXP_0		0x0000	  // Experimental 0
 #define	USB_PID_FSF_EXP_1		0x0001	  // Experimental 1
 #define	USB_PID_FSF_USRP		0x0002	  // Universal Software Radio Peripheral
+#define	USB_PID_FSF_USRP_reserved	0x0003	  // Universal Software Radio Peripheral
+#define	USB_PID_FSF_SSRP		0x0004	  // Simple Software Radio Peripheral
+#define	USB_PID_FSF_SSRP_reserved	0x0005	  // Simple Software Radio Peripheral
 
 #define	USB_DID_USRP_0			0x0000	  // unconfigured rev 0 USRP
 #define	USB_DID_USRP_1			0x0001	  // unconfigured rev 1 USRP
+#define	USB_DID_USRP_2			0x0002	  // unconfigured rev 2 USRP
 
 #endif /* _USRP_IDS_H_ */
