@@ -526,7 +526,7 @@ fusb_ephandle_linux::read (void *buffer, int nbytes)
       if (!reload_read_buffer ())
 	return -1;
 
-    int m = std::min (nbytes - n, d_read_buffer_end - d_read_buffer);
+    int m = std::min (nbytes - n, (int) (d_read_buffer_end - d_read_buffer));
 
     memcpy (&dst[n], d_read_buffer, m);
     d_read_buffer += m;
