@@ -109,6 +109,8 @@ usrp_basic::usrp_basic (int which_board,
     d_bytes_per_poll ((int) (POLLING_INTERVAL * d_usb_data_rate)),
     d_verbose (false)
 {
+  memset (d_fpga_shadows, 0, sizeof (d_fpga_shadows));
+
   usrp_one_time_init ();
 
   // FIXME allow subclasses to load own code
@@ -200,6 +202,112 @@ usrp_basic::read_eeprom (int i2c_addr, int eeprom_offset, int len)
   return std::string (buf, len);
 }
 
+bool
+usrp_basic::write_i2c (int i2c_addr, const std::string buf)
+{
+  return usrp_i2c_write (d_udh, i2c_addr, buf.data (), buf.size ());
+}
+
+std::string
+usrp_basic::read_i2c (int i2c_addr, int len)
+{
+  if (len <= 0)
+    return "";
+
+  char buf[len];
+
+  if (!usrp_i2c_read (d_udh, i2c_addr, buf, len))
+    return "";
+
+  return std::string (buf, len);
+}
+
+// ----------------------------------------------------------------
+
+bool
+usrp_basic::set_adc_offset (int which, int offset)
+{
+  int	r, v;
+
+  switch (which){
+  case 0:
+    r = FR_ADC_OFFSET_1_0;
+    v = (d_fpga_shadows[r] & 0xffff0000) | (offset & 0xffff);
+    break;
+  case 1:
+    r = FR_ADC_OFFSET_1_0;
+    v = (d_fpga_shadows[r] & 0x0000ffff) | ((offset & 0xffff) << 16);
+    break;
+  case 2:
+    r = FR_ADC_OFFSET_3_2;
+    v = (d_fpga_shadows[r] & 0xffff0000) | (offset & 0xffff);
+    break;
+  case 3:
+    r = FR_ADC_OFFSET_3_2;
+    v = (d_fpga_shadows[r] & 0x0000ffff) | ((offset & 0xffff) << 16);
+    break;
+  default:
+    return false;
+  }
+  return _write_fpga_reg (r, v);
+}
+
+bool
+usrp_basic::set_dac_offset (int which, int offset, int offset_pin)
+{
+  if (which < 0 || which > 3)
+    return false;
+
+  int which_codec = which >> 1;
+  int tx_a = (which & 0x1) == 0;
+  int lo = ((offset & 0x3) << 6) | (offset_pin & 0x1);
+  int hi = (offset >> 2);
+  bool ok;
+
+  if (tx_a){
+    ok =  _write_9862 (which_codec, REG_TX_A_OFFSET_LO, lo);
+    ok &= _write_9862 (which_codec, REG_TX_A_OFFSET_HI, hi);
+  }
+  else {
+    ok =  _write_9862 (which_codec, REG_TX_B_OFFSET_LO, lo);
+    ok &= _write_9862 (which_codec, REG_TX_B_OFFSET_HI, hi);
+  }
+  return ok;
+}
+
+bool
+usrp_basic::set_adc_buffer_bypass (int which, bool bypass)
+{
+  if (which < 0 || which > 3)
+    return false;
+
+  int codec = which >> 1;
+  int reg = (which & 1) == 0 ? REG_RX_A : REG_RX_B;
+
+  unsigned char cur_rx;
+  unsigned char cur_pwr_dn;
+
+  // If the input buffer is bypassed, we need to power it down too.
+
+  bool ok = _read_9862 (codec, reg, &cur_rx);
+  ok &= _read_9862 (codec, REG_RX_PWR_DN, &cur_pwr_dn);
+  if (!ok)
+    return false;
+
+  if (bypass){
+    cur_rx |= RX_X_BYPASS_INPUT_BUFFER;
+    cur_pwr_dn |= ((which & 1) == 0) ? RX_PWR_DN_BUF_A : RX_PWR_DN_BUF_B;
+  }
+  else {
+    cur_rx &= ~RX_X_BYPASS_INPUT_BUFFER;
+    cur_pwr_dn &= ~(((which & 1) == 0) ? RX_PWR_DN_BUF_A : RX_PWR_DN_BUF_B);
+  }
+
+  ok &= _write_9862 (codec, reg, cur_rx);
+  ok &= _write_9862 (codec, REG_RX_PWR_DN, cur_pwr_dn);
+  return ok;
+}
+
 // ----------------------------------------------------------------
 
 bool
@@ -209,6 +317,10 @@ usrp_basic::_write_fpga_reg (int regno, int value)
     fprintf (stdout, "_write_fpga_reg (%3d, 0x%08x)\n", regno, value);
     fflush (stdout);
   }
+
+  if (regno >= 0 && regno < MAX_REGS)
+    d_fpga_shadows[regno] = value;
+
   return usrp_write_fpga_reg (d_udh, regno, value);
 }
 
@@ -469,9 +581,15 @@ usrp_basic_rx::set_pga (int which, double gain)
   int codec = which >> 1;
   int reg = (which & 1) == 0 ? REG_RX_A : REG_RX_B;
 
+  // read current value to get input buffer bypass flag.
+  unsigned char cur_rx;
+  if (!_read_9862 (codec, reg, &cur_rx))
+    return false;
+
   int int_gain = (int) rint ((gain - pga_min ()) / pga_db_per_step());
 
-  return _write_9862 (codec, reg, int_gain);
+  cur_rx = (cur_rx & RX_X_BYPASS_INPUT_BUFFER) | (int_gain & 0x7f);
+  return _write_9862 (codec, reg, cur_rx);
 }
 
 double
@@ -516,15 +634,14 @@ usrp_basic_rx::probe_rx_slots (bool verbose)
   for (int i = 0; i < 2; i++){
     int slot_id = slot_id_map [i];
     const char *msg = 0;
-    unsigned offset;
     usrp_dbeeprom_status_t s = usrp_read_dboard_eeprom (d_udh, slot_id, &eeprom);
 
     switch (s){
     case UDBE_OK:
       d_dbid[i] = eeprom.id;
       msg = usrp_dbid_to_string (eeprom.id).c_str ();
-      offset = (eeprom.offset[1] << 16) | (eeprom.offset[0] & 0xffff);
-      _write_fpga_reg (i == 0 ? FR_ADC_OFFSET_1_0 : FR_ADC_OFFSET_3_2, offset);
+      set_adc_offset (2*i+0, eeprom.offset[0]);
+      set_adc_offset (2*i+1, eeprom.offset[1]);
       _write_fpga_reg (slot_id_to_oe_reg(slot_id), (0xffff << 16) | eeprom.oe);
       break;
       
@@ -597,6 +714,26 @@ usrp_basic_rx::read_io (int which_dboard)
   return value;
 }
 
+bool
+usrp_basic_rx::write_aux_dac (int which_dboard, int which_dac, int value)
+{
+  return usrp_basic::write_aux_dac (dboard_to_slot (which_dboard),
+				    which_dac, value);
+}
+
+bool
+usrp_basic_rx::read_aux_adc (int which_dboard, int which_adc, int *value)
+{
+  return usrp_basic::read_aux_adc (dboard_to_slot (which_dboard),
+				   which_adc, value);
+}
+
+int
+usrp_basic_rx::read_aux_adc (int which_dboard, int which_adc)
+{
+  return usrp_basic::read_aux_adc (dboard_to_slot (which_dboard), which_adc);
+}
+
 ////////////////////////////////////////////////////////////////
 //
 //			   usrp_basic_tx
@@ -935,3 +1072,24 @@ usrp_basic_tx::read_io (int which_dboard)
     return READ_FAILED;
   return value;
 }
+
+bool
+usrp_basic_tx::write_aux_dac (int which_dboard, int which_dac, int value)
+{
+  return usrp_basic::write_aux_dac (dboard_to_slot (which_dboard),
+				    which_dac, value);
+}
+
+bool
+usrp_basic_tx::read_aux_adc (int which_dboard, int which_adc, int *value)
+{
+  return usrp_basic::read_aux_adc (dboard_to_slot (which_dboard),
+				   which_adc, value);
+}
+
+int
+usrp_basic_tx::read_aux_adc (int which_dboard, int which_adc)
+{
+  return usrp_basic::read_aux_adc (dboard_to_slot (which_dboard), which_adc);
+}
+
