@@ -63,6 +63,9 @@ protected:
   int			 d_bytes_per_poll;	// how often to poll for overruns
   bool			 d_verbose;
 
+  static const int	 MAX_REGS = 128;
+  unsigned int		 d_fpga_shadows[MAX_REGS];
+
   usrp_basic (int which_board,
 	      struct usb_dev_handle *open_interface (struct usb_device *dev));
 
@@ -82,24 +85,6 @@ protected:
    */
   void set_usb_data_rate (int usb_data_rate);
   
-public:
-  virtual ~usrp_basic ();
-
-  /*!
-   * \brief return frequency of master oscillator on USRP
-   */
-  long  fpga_master_clock_freq () const { return 128000000; }
-
-  /*!
-   * \returns usb data rate in bytes/sec
-   */
-  int usb_data_rate () const { return d_usb_data_rate; }
-
-  void set_verbose (bool on) { d_verbose = on; }
-
-  //! magic value used on alternate register read interfaces
-  static const int READ_FAILED = -99999;
-
   /*!
    * \brief Write auxiliary digital to analog converter.
    *
@@ -131,6 +116,24 @@ public:
    */
   int read_aux_adc (int slot, int which_adc);
 
+public:
+  virtual ~usrp_basic ();
+
+  /*!
+   * \brief return frequency of master oscillator on USRP
+   */
+  long  fpga_master_clock_freq () const { return 128000000; }
+
+  /*!
+   * \returns usb data rate in bytes/sec
+   */
+  int usb_data_rate () const { return d_usb_data_rate; }
+
+  void set_verbose (bool on) { d_verbose = on; }
+
+  //! magic value used on alternate register read interfaces
+  static const int READ_FAILED = -99999;
+
   /*!
    * \brief Write EEPROM on motherboard or any daughterboard.
    * \param i2c_addr		I2C bus address of EEPROM
@@ -141,7 +144,7 @@ public:
   bool write_eeprom (int i2c_addr, int eeprom_offset, const std::string buf);
 
   /*!
-   * \brief Write EEPROM on motherboard or any daughterboard.
+   * \brief Read EEPROM on motherboard or any daughterboard.
    * \param i2c_addr		I2C bus address of EEPROM
    * \param eeprom_offset	byte offset in EEPROM to begin reading
    * \param len			number of bytes to read
@@ -149,6 +152,48 @@ public:
    */
   std::string read_eeprom (int i2c_addr, int eeprom_offset, int len);
 
+  /*!
+   * \brief Write to I2C peripheral
+   * \param i2c_addr		I2C bus address (7-bits)
+   * \param buf			the data to write
+   * \returns true iff successful
+   * Writes are limited to a maximum of of 64 bytes.
+   */
+  bool write_i2c (int i2c_addr, const std::string buf);
+
+  /*!
+   * \brief Read from I2C peripheral
+   * \param i2c_addr		I2C bus address (7-bits)
+   * \param len			number of bytes to read
+   * \returns the data read if successful, else a zero length string.
+   * Reads are limited to a maximum of of 64 bytes.
+   */
+  std::string read_i2c (int i2c_addr, int len);
+
+  /*!
+   * \brief Set ADC offset correction
+   * \param which	which ADC[0,3]: 0 = RX_A I, 1 = RX_A Q...
+   * \param offset	16-bit value to subtract from raw ADC input.
+   */
+  bool set_adc_offset (int which, int offset);
+
+  /*!
+   * \brief Set DAC offset correction
+   * \param which	which DAC[0,3]: 0 = TX_A I, 1 = TX_A Q...
+   * \param offset	10-bit offset value (ambiguous format:  See AD9862 datasheet).
+   * \param offset_pin	1-bit value.  If 0 offset applied to -ve differential pin;
+   *                                  If 1 offset applied to +ve differential pin.
+   */
+  bool set_dac_offset (int which, int offset, int offset_pin);
+
+  /*!
+   * \brief Control ADC input buffer
+   * \param which 	which ADC[0,3]
+   * \param bypass	if non-zero, bypass input buffer and connect input
+   *	                directly to switched cap SHA input of RxPGA.
+   */
+  bool set_adc_buffer_bypass (int which, bool bypass);
+
   // ----------------------------------------------------------------
   // Low level implementation routines.
   // You probably shouldn't be using these...
@@ -362,6 +407,37 @@ public:
    * \returns register value if successful, else READ_FAILED
    */
   int read_io (int which_dboard);
+
+  /*!
+   * \brief Write auxiliary digital to analog converter.
+   *
+   * \param which_dboard	[0,1] which d'board
+   *    			N.B., SLOT_TX_A and SLOT_RX_A share the same AUX DAC's.
+   *          			SLOT_TX_B and SLOT_RX_B share the same AUX DAC's.
+   * \param which_dac		[2,3] TX slots must use only 2 and 3.
+   * \param value		[0,4095]
+   * \returns true iff successful
+   */
+  bool write_aux_dac (int which_board, int which_dac, int value);
+
+  /*!
+   * \brief Read auxiliary analog to digital converter.
+   *
+   * \param which_dboard	[0,1] which d'board
+   * \param which_adc		[0,1]
+   * \param value		return 12-bit value [0,4095]
+   * \returns true iff successful
+   */
+  bool read_aux_adc (int which_dboard, int which_adc, int *value);
+
+  /*!
+   * \brief Read auxiliary analog to digital converter.
+   *
+   * \param which_dboard	[0,1] which d'board
+   * \param which_adc		[0,1]
+   * \returns value in the range [0,4095] if successful, else READ_FAILED.
+   */
+  int read_aux_adc (int which_dboard, int which_adc);
 };
 
 /*!
@@ -528,5 +604,37 @@ public:
    * \returns register value if successful, else READ_FAILED
    */
   int read_io (int which_dboard);
+
+  /*!
+   * \brief Write auxiliary digital to analog converter.
+   *
+   * \param which_dboard	[0,1] which d'board
+   *    			N.B., SLOT_TX_A and SLOT_RX_A share the same AUX DAC's.
+   *          			SLOT_TX_B and SLOT_RX_B share the same AUX DAC's.
+   * \param which_dac		[2,3] TX slots must use only 2 and 3.
+   * \param value		[0,4095]
+   * \returns true iff successful
+   */
+  bool write_aux_dac (int which_board, int which_dac, int value);
+
+  /*!
+   * \brief Read auxiliary analog to digital converter.
+   *
+   * \param which_dboard	[0,1] which d'board
+   * \param which_adc		[0,1]
+   * \param value		return 12-bit value [0,4095]
+   * \returns true iff successful
+   */
+  bool read_aux_adc (int which_dboard, int which_adc, int *value);
+
+  /*!
+   * \brief Read auxiliary analog to digital converter.
+   *
+   * \param which_dboard	[0,1] which d'board
+   * \param which_adc		[0,1]
+   * \returns value in the range [0,4095] if successful, else READ_FAILED.
+   */
+  int read_aux_adc (int which_dboard, int which_adc);
 };
+
 #endif
