@@ -29,6 +29,8 @@
 
 #define	USRP_DBID_BASIC_TX		0x0000
 #define	USRP_DBID_BASIC_RX		0x0001
+#define USRP_DBID_DBS_RX		0x0002
+#define USRP_DBID_TV_RX			0x0003
 #define	USRP_DBID_EXPERIMENTAL_TX	0xfffe	// homebrew, etc
 #define	USRP_DBID_EXPERIMENTAL_RX	0xffff
 
