@@ -32,6 +32,8 @@ static struct {
 } dbid_map[] = {
   { USRP_DBID_BASIC_TX,		"Basic Tx" },
   { USRP_DBID_BASIC_RX,		"Basic Rx" },
+  { USRP_DBID_DBS_RX,		"DBS Rx" },
+  { USRP_DBID_TV_RX,		"TV Rx" },
   { USRP_DBID_EXPERIMENTAL_TX,	"Experimental Tx" },
   { USRP_DBID_EXPERIMENTAL_RX,	"Experimental Rx" }
 };
