@@ -38,6 +38,11 @@
 #include <time.h>		// FIXME should check with autoconf (nanosleep)
 #include <algorithm>
 #include <ad9862.h>
+#include <assert.h>
+
+extern "C" {
+#include "md5.h"
+};
 
 #define VERBOSE 0
 
@@ -54,11 +59,7 @@ static const int hash_slot_addr[2] = {
 static char *usrp_firmware_filename = "usrp_firmware.ihx";
 static char *usrp_fpga_filename = "usrp_fpga.rbf";
 
-static char *std_paths[] = {
-  "/usr/local/share/usrp",
-  0
-};
-
+#include "std_paths.h"
 
 static char *
 find_file (const char *filename, int hw_rev)
@@ -658,60 +659,21 @@ usrp_set_sleep_bits (struct usb_dev_handle *udh, int bits, int mask)
 // ----------------------------------------------------------------
 // conditional load stuff
 
-static int
-hexval (unsigned char ch)
-{
-  if ('0' <= ch && ch <= '9')
-    return ch - '0';
-
-  if ('a' <= ch && ch <= 'f')
-    return ch - 'a' + 10;
-
-  if ('A' <= ch && ch <= 'F')
-    return ch - 'A' + 10;
-
-  return -1;
-}
-
 static bool
 compute_hash (const char *filename, unsigned char hash[USRP_HASH_SIZE])
 {
+  assert (USRP_HASH_SIZE == 16);
   memset (hash, 0, USRP_HASH_SIZE);
 
-  char cmd[1000];
-  unsigned char line[32];
+  FILE *fp = fopen (filename, "rb");
+  if (fp == 0){
+    perror (filename);
+    return false;
+  }
+  int r = md5_stream (fp, hash);
+  fclose (fp);
   
-  snprintf (cmd, sizeof (cmd), "md5sum %s", filename);
-  FILE *fp = popen (cmd, "r");
-  if (fp == 0)
-    goto fail;
-  
-  if (fread (line, 1, 2*USRP_HASH_SIZE, fp) != 2*USRP_HASH_SIZE){
-    perror ("usrp_prims: fread of md5sum pipe");
-    goto fail;
-  }
-
-  if (pclose (fp) != 0){
-    fp = 0;
-    goto fail;
-  }
-
-  for (int i = 0; i < USRP_HASH_SIZE; i++){
-    int	hi = hexval (line[2*i + 0]);
-    int lo = hexval (line[2*i + 1]);
-
-    if (hi < 0 || lo < 0)
-      goto fail;
-
-    hash[i] = (hi << 4) | lo;
-  }
-
-  return true;
-
- fail:
-  if (fp)
-    pclose (fp);
-  return false;
+  return r == 0;
 }
 
 static usrp_load_status_t
