@@ -167,6 +167,15 @@ usrp_standard_rx::make (int which_board,
 bool
 usrp_standard_rx::set_decim_rate  (unsigned int rate)
 {
+  if (rate < 1 || rate > 256){
+    fprintf (stderr, "usrp_standard_rx::set_decim_rate: rate must be in [1, 256]\n");
+    return false;
+  }
+
+  if (rate > 128){
+    fprintf (stderr, "usrp_standard_rx::set_decim_rate: WARNING rates > 128 result in incorrect gain.\n");
+  }
+
   d_decim_rate = rate;
   set_usb_data_rate ((adc_freq () / rate * nchannels ())
 		     * (2 * sizeof (short)));
@@ -434,8 +443,8 @@ usrp_standard_tx::set_interp_rate (unsigned int rate)
 {
   // fprintf (stderr, "usrp_standard_tx::set_interp_rate\n");
 
-  if (rate & 0x3){
-    fprintf (stderr, "usrp_standard_tx::set_interp_rate: rate must be a multiple of 4!\n");
+  if ((rate & 0x3) || rate < 4 || rate > 512){
+    fprintf (stderr, "usrp_standard_tx::set_interp_rate: rate must be in [4, 512] and a multiple of 4.\n");
     return false;
   }
 
